# Generated by Django 5.0 on 2024-01-09 20:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('easy_docs', '0002_alter_documentation_reference_url_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentation',
            name='keywords',
            field=models.TextField(blank=True, help_text='Comma separated list of keywords to help users find this documentation.'),
        ),
        migrations.AlterField(
            model_name='historicaldocumentation',
            name='keywords',
            field=models.TextField(blank=True, help_text='Comma separated list of keywords to help users find this documentation.'),
        ),
    ]



from django.db import models
from markdownx.models import MarkdownxField
from simple_history.models import HistoricalRecords
from django.utils.text import slugify
from django.core.exceptions import ValidationError
from django.conf import settings
from .middleware import CurrentUserMiddleware


class Documentation(models.Model):
    title = models.CharField(max_length=100, unique=True)
    title_slug = models.SlugField(max_length=100, unique=True)
    reference_url = models.CharField(max_length=200, unique=True, blank=True, null=True)
    content = MarkdownxField()
    keywords = models.TextField(null=False, blank=True, help_text="Comma separated list of keywords to help users find this documentation.")
    public = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    updated_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, related_name='updated_documentations')
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, related_name='created_documentations')
    history = HistoricalRecords()

    def save(self, *args, **kwargs):
        current_user = CurrentUserMiddleware.get_current_user()
        if current_user and not current_user.is_anonymous:
            self.updated_by = current_user
        if not self.title_slug:
            self.title_slug = slugify(self.title)[:100]
            if current_user and not current_user.is_anonymous:
                self.created_by = current_user
        super(Documentation, self).save(*args, **kwargs)

    def clean(self):
        if self.reference_url:
            existing = Documentation.objects.filter(reference_url=self.reference_url).exclude(pk=self.pk)
            if existing.exists():
                raise ValidationError("A Documentation with this reference URL already exists.")
        super(Documentation, self).clean()

    def __str__(self):
        return self.title


from django.contrib import admin
from simple_history.admin import SimpleHistoryAdmin
from .models import Documentation

# Register your models here.

admin.site.register(Documentation, SimpleHistoryAdmin)


from django import forms
from .models import Documentation
from markdownx.fields import MarkdownxFormField

class DocumentationForm(forms.ModelForm):
    title = forms.CharField(max_length=100, required=True, widget=forms.TextInput(attrs={'class': 'form-control form-control-lg'}), help_text="The title of this documentation.")
    content = MarkdownxFormField()
    keywords = forms.CharField(max_length=100, required=False, widget=forms.TextInput(attrs={'class': 'form-control form-control-lg'}), help_text="Comma separated list of keywords to help users find this documentation.", empty_value="Comma seperated list.")
    public = forms.BooleanField(required=False, widget=forms.CheckboxInput(attrs={'class': 'form-check-input'}), help_text="Check this box to make this documentation public.")
    reference_url = forms.CharField(max_length=200, required=False, widget=forms.HiddenInput())

    class Meta:
        model = Documentation
        fields = ('title', 'content', 'keywords', 'public', 'reference_url')

class ManualDocumentationForm(forms.ModelForm):
    title = forms.CharField(max_length=100, required=True, widget=forms.TextInput(attrs={'class': 'form-control form-control-lg'}), help_text="The title of this documentation.")
    content = MarkdownxFormField()
    keywords = forms.CharField(max_length=100, required=False, widget=forms.TextInput(attrs={'class': 'form-control form-control-lg'}), help_text="Comma separated list of keywords to help users find this documentation.", empty_value="Comma seperated list.")
    public = forms.BooleanField(required=False, widget=forms.CheckboxInput(attrs={'class': 'form-check-input'}), help_text="Check this box to make this documentation public.")
    reference_url = forms.CharField(max_length=200, required=False, help_text="The URL of the page you are documenting, if any.", widget=forms.TextInput(attrs={'class': 'form-control form-control-lg'}))

    class Meta:
        model = Documentation
        fields = ('title', 'content', 'keywords', 'public', 'reference_url')



from threading import local

_user_local = local()

class CurrentUserMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        _user_local.user = request.user
        try:
            response = self.get_response(request)
        finally:
            if hasattr(_user_local, 'user'):
                del _user_local.user
        return response

    @classmethod
    def get_current_user(cls):
        return getattr(_user_local, 'user', None)


# Generated by Django 5.0 on 2024-01-08 18:52

import django.db.models.deletion
import markdownx.models
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Documentation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, unique=True)),
                ('title_slug', models.SlugField(max_length=100, unique=True)),
                ('reference_url', models.URLField(blank=True, null=True, unique=True)),
                ('content', markdownx.models.MarkdownxField()),
                ('keywords', models.TextField(blank=True)),
                ('public', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='created_documentations', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='updated_documentations', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='HistoricalDocumentation',
            fields=[
                ('id', models.BigIntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('title', models.CharField(db_index=True, max_length=100)),
                ('title_slug', models.SlugField(max_length=100)),
                ('reference_url', models.URLField(blank=True, db_index=True, null=True)),
                ('content', markdownx.models.MarkdownxField()),
                ('keywords', models.TextField(blank=True)),
                ('public', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(blank=True, editable=False)),
                ('updated_at', models.DateTimeField(blank=True, editable=False)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField(db_index=True)),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('created_by', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical documentation',
                'verbose_name_plural': 'historical documentations',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': ('history_date', 'history_id'),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from django.shortcuts import render
from .models import Documentation
from urllib.parse import unquote
from django.contrib.admin.views.decorators import staff_member_required
from .forms import DocumentationForm, ManualDocumentationForm
from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.views.generic import ListView, DetailView
from django.http import Http404
from django.core.paginator import Paginator
from django.db.models import Q

def get_document(request, page_url):
    decoded_url = unquote(page_url)
    documentation = Documentation.objects.filter(reference_url=decoded_url)
    if documentation.exists():
        documentation = documentation.first()
        if documentation.public or request.user.is_staff:
            return render(request, 'doc_modal.html', {'documentation': documentation})
    else:
        if request.user.is_staff:
            form = DocumentationForm()
            return render(request, 'create_documentation.html', {'url_ref': decoded_url, 'form': form})
    return render(request, 'not_found.html')

@staff_member_required
def create_documentation(request):
    if request.method == 'POST':
        form = DocumentationForm(request.POST, initial={'reference_url': request.POST.get('reference_url')})
        if form.is_valid():
            documentation = form.save()
            messages.success(request, 'Documentation created successfully!')
            return render(request, 'doc_modal.html', {'documentation': documentation})
        else:
            return render(request, 'create_documentation.html', {'form': form})
    else:
        form = DocumentationForm(initial={'reference_url': request.GET.get('url_ref')})
        return render(request, 'create_documentation.html', {'form': form})
    
@staff_member_required
def edit_documentation(request, pk):
    documentation = get_object_or_404(Documentation, pk=pk)
    slug_view = request.GET.get('slug_view', False)
    context = {'documentation': documentation, 'slug_view': slug_view}
    if slug_view:
        template = 'edit_documentation_from_slug.html'
    else:
        template = 'edit_documentation.html'
    success_template = 'doc_modal.html'
    if request.method == 'POST':
        form = DocumentationForm(request.POST, instance=documentation)
        if form.is_valid():
            form.save()
            messages.success(request, 'Documentation updated successfully!')
            return render(request, success_template, context)
        else:
            context['form'] = form
            return render(request, template, context)
    else:
        form = DocumentationForm(instance=documentation)
        context['form'] = form
        return render(request, template, context)
    
class AllDocumentation(ListView):
    model = Documentation
    template_name = 'all_documentation.html'
    context_object_name = 'documents'
    ordering = ['-updated_at']
    paginate_by = 10

    def get_template_names(self):
        if self.request.htmx:
            return 'documentation_list.html'
        return 'all_documentation.html'
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["url"] = self.request.path
        return context
    
    def get_queryset(self):
        queryset = super().get_queryset()
        if not self.request.user.is_staff:
            queryset = queryset.filter(public=True)
        search_query = self.request.GET.get('search', '').strip()
        if search_query:
            search_keywords = search_query.split(',')
            query = Q()
            for keyword in search_keywords:
                query |= Q(keywords__icontains=keyword.strip())
            queryset = queryset.filter(query)
        return queryset
    
def view_document(request, slug):
    documentation = get_object_or_404(Documentation, title_slug=slug)
    if documentation.public or request.user.is_staff:
        if request.htmx:
            return render(request, 'doc_modal.html', {'documentation': documentation})
        return render(request, 'view_documentation.html', {'documentation': documentation, 'slug_view' : True})
    else:
        raise Http404

@staff_member_required
def add_documentation(request):
    if request.method == 'POST':
        form = ManualDocumentationForm(request.POST)
        if form.is_valid():
            documentation = form.save()
            messages.success(request, 'Documentation created successfully!')
            return render(request, 'doc_modal.html', {'documentation': documentation})
        else:
            return render(request, 'create_documentation.html', {'form': form})
    else:
        form = ManualDocumentationForm()
        return render(request, 'create_documentation.html', {'form': form})


class HistoryView(DetailView):
    model = Documentation
    template_name = 'view_history.html'
    context_object_name = 'documentation'
    paginate_by = 10

    def get_template_names(self):
        if self.request.htmx:
            return 'history_list.html'
        return 'view_history.html'

    def get_object(self):
        obj = super().get_object()
        if not self.request.user.is_staff:
            raise Http404
        return obj
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["url"] = self.request.path
        history_objects = context['documentation'].history.all().order_by('-history_date')
        paginator = Paginator(history_objects, self.paginate_by)
        page_number = self.request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        context['history'] = page_obj
        context['is_paginated'] = page_obj.has_other_pages()
        context['page_obj'] = page_obj
        return context

@staff_member_required
def revert_history(request, pk, history_id):
    documentation = get_object_or_404(Documentation, pk=pk)
    history_record = documentation.history.get(pk=history_id)
    for field in history_record.instance._meta.fields:
        setattr(documentation, field.name, getattr(history_record, field.name))
    documentation.save()
    messages.success(request, 'Documentation reverted successfully!')
    return render(request, 'doc_modal.html', {'documentation': documentation})

@staff_member_required
def view_history_document(request, pk, history_id):
    documentation = get_object_or_404(Documentation, pk=pk)
    history = documentation.history.get(pk=history_id)
    return render(request, 'doc_modal.html', {'documentation': history.instance, 'history_view' : True, 'history_id' : history_id})